package com.eatpoint.plugins;

import android.Manifest;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.animation.BounceInterpolator;
import android.widget.TextView;

import com.eatpoint.plugins.api.ApiResult;
import com.eatpoint.plugins.api.ApiService;
import com.eatpoint.plugins.floatwindow.FloatWindow;
import com.eatpoint.plugins.floatwindow.IFloatWindow;
import com.eatpoint.plugins.floatwindow.MoveType;
import com.eatpoint.plugins.floatwindow.PermissionListener;
import com.eatpoint.plugins.floatwindow.Screen;
import com.eatpoint.plugins.floatwindow.Util;
import com.eatpoint.plugins.floatwindow.ViewStateListener;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;

import static android.app.Activity.RESULT_OK;

/**
 * This class echoes a string called from JavaScript.
 */
public class CallPhoneFloatWindow extends CordovaPlugin {

    private static final String TAG = "FloatWindow";
    private static final String TAG_MIN = "FloatWindow_min";
    private static final String TAG_OPEN = "FloatWindow_open";
    private UserEntity userEntity = new UserEntity();

    private FakeR fakeR;
    private Retrofit retrofit;
    private boolean isFirstListener = true;

    @Override
    public void onStart() {
        super.onStart();
        fakeR = new FakeR(cordova.getActivity());
    }

    @Override
    public boolean execute(String action, String rawArgs, CallbackContext callbackContext) {
        fakeR = new FakeR(cordova.getActivity());
        if (action.equals("showFW")) {
            cordova.getActivity().runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    userEntity = new UserEntity(rawArgs);
                    showFW(userEntity, callbackContext);
                }
            });
            return true;
        } else if (action.equals("startCallListener")) {
            cordova.getActivity().runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    checkPermissions();
//                    queryUserInfo("17687226750");
                    callbackContext.success();
                }
            });
        }
        return false;
    }

    public CallPhoneFloatWindow getInstance() {
        return this;
    }

    public void ShowFW(UserEntity userEntity) {
        showFW(userEntity, null);
    }

    private void showFW(UserEntity userEntity, CallbackContext callbackContext) {

        if (FloatWindow.get(TAG_MIN) == null) {
            createFloatWindowMin();
        }
        showOneFloatWindow(true);
        TextView tx = Objects.requireNonNull(FloatWindow.get(TAG_MIN)).getView().findViewById(fakeR.getId("id", "tx_info"));
        tx.setText(String.format("%s\n%s", userEntity.getCustomer_name(), userEntity.getMobile_phone()));
        if (callbackContext != null) {
            callbackContext.success("success");
        }
    }

    private void createFloatWindowMin() {

        View view = LayoutInflater.from(this.cordova.getContext()).inflate(fakeR.getId("layout", "view_float_window"), null);
        view.findViewById(fakeR.getId("id", "btn_open")).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                createFloatWindowOpen(userEntity);
                showOneFloatWindow(false);
            }
        });
        view.findViewById(fakeR.getId("id", "btn_exit")).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Objects.requireNonNull(FloatWindow.get(TAG_MIN)).hide();
            }
        });

        FloatWindow
                .with(this.cordova.getActivity().getApplicationContext())
                .setTag(TAG_MIN)
                .setView(view)
                .setWidth(Screen.width, 0.45f) //设置悬浮控件宽高
                .setHeight(Screen.width, 0.30f)
                .setX(Screen.width, 0.55f)
                .setY(Screen.height, 0.05f)
                .setMoveType(MoveType.slide, 0, 0)
                .setMoveStyle(500, new BounceInterpolator())
                .setFilter(false)
                .setViewStateListener(mViewStateListener)
                .setPermissionListener(mPermissionListener)
                .setDesktopShow(true)
                .build();
    }

    private void createFloatWindowOpen(UserEntity userEntity) {

        View view;
        if (FloatWindow.get(TAG_OPEN) == null) {
            view = LayoutInflater.from(this.cordova.getContext()).inflate(fakeR.getId("layout", "view_float_window_open"), null);
            FloatWindow
                    .with(this.cordova.getActivity().getApplicationContext())
                    .setTag(TAG_OPEN)
                    .setView(view)
                    .setWidth(Screen.width, 0.9f) //设置悬浮控件宽高
                    .setHeight(Util.dp2px(cordova.getContext(), 290))
                    .setX(Screen.width, 0.05f)
                    .setY(Screen.height, 0.05f)
                    .setMoveType(MoveType.inactive)
                    .setViewStateListener(mViewStateListener)
                    .setPermissionListener(mPermissionListener)
                    .setDesktopShow(true)
                    .build();

            TextView btnClose = view.findViewById(fakeR.getId("id", "btn_close"));
            btnClose.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    showOneFloatWindow(true);
                }
            });
        }

        view = Objects.requireNonNull(FloatWindow.get(TAG_OPEN)).getView();
        TextView txName = view.findViewById(fakeR.getId("id", "tx_name"));
        TextView txSexAndAge = view.findViewById(fakeR.getId("id", "tx_sex_and_age"));
        TextView txPhone = view.findViewById(fakeR.getId("id", "tx_phone"));
        TextView txConsultant = view.findViewById(fakeR.getId("id", "tx_consultant"));
        TextView txCreateTime = view.findViewById(fakeR.getId("id", "tx_create_time"));
        TextView txLevel = view.findViewById(fakeR.getId("id", "tx_level"));
        TextView txCustomerSource = view.findViewById(fakeR.getId("id", "tx_customer_source"));
        TextView txCarModel = view.findViewById(fakeR.getId("id", "tx_car_model"));

        txName.setText(userEntity.getCustomer_name());
        txPhone.setText(userEntity.getMobile_phone());
        txLevel.setText(userEntity.getLevel());
        txCarModel.setText(userEntity.getModel_name());
        txSexAndAge.setText(String.valueOf(userEntity.getAge()));
        txConsultant.setText(userEntity.getTelemarketing_no_name());
        txCreateTime.setText(userEntity.getCreated_at());
        txCustomerSource.setText(userEntity.getAll_label());

    }

    private void showOneFloatWindow(boolean isMin) {
        IFloatWindow min = FloatWindow.get(TAG_MIN);
        IFloatWindow open = FloatWindow.get(TAG_OPEN);
        if (isMin) {
            if (min != null) min.show();
            if (open != null) open.hide();
        } else {
            if (min != null) min.hide();
            if (open != null) open.show();
        }
    }

    private PermissionListener mPermissionListener = new PermissionListener() {
        @Override
        public void onSuccess() {
            Log.d(TAG, "onSuccess");
        }

        @Override
        public void onFail() {
            Log.d(TAG, "onFail");
        }
    };

    private ViewStateListener mViewStateListener = new ViewStateListener() {
        @Override
        public void onPositionUpdate(int x, int y) {
            Log.d(TAG, "onPositionUpdate: x=" + x + " y=" + y);
        }

        @Override
        public void onShow() {
            Log.d(TAG, "onShow");
        }

        @Override
        public void onHide() {
            Log.d(TAG, "onHide");
        }

        @Override
        public void onDismiss() {
            Log.d(TAG, "onDismiss");
        }

        @Override
        public void onMoveAnimStart() {
            Log.d(TAG, "onMoveAnimStart");
        }

        @Override
        public void onMoveAnimEnd() {
            Log.d(TAG, "onMoveAnimEnd");
        }

        @Override
        public void onBackToDesktop() {
            Log.d(TAG, "onBackToDesktop");
        }
    };

    private Retrofit getRetrofit() {
        if (retrofit == null) {
            synchronized (CallPhoneFloatWindow.class) {
                retrofit = new Retrofit.Builder()
                        .baseUrl("https://foo.baojunev.com/mock/122/")
                        .addConverterFactory(GsonConverterFactory.create())
                        .build();
            }
        }
        return retrofit;
    }

    private void queryUserInfo(String phone) {
        ApiService apiService = getRetrofit().create(ApiService.class);
        Call<ApiResult<UserEntity>> call = apiService.getUserInfo(phone);
        call.enqueue(new Callback<ApiResult<UserEntity>>() {
            @Override
            public void onResponse(Call<ApiResult<UserEntity>> call, Response<ApiResult<UserEntity>> response) {
                if (response.isSuccessful() && response.body() != null && response.body().getData() != null) {
                    if (response.body().getCode() == 200) {
                        userEntity = response.body().getData();
                        ShowFW(userEntity);
                    }
                }

            }

            @Override
            public void onFailure(Call<ApiResult<UserEntity>> call, Throwable t) {

            }
        });
    }

    /**
     * 获取来电监听权限
     */
    private void checkPermissions() {
        String[] permissions = {Manifest.permission.READ_CALL_LOG, Manifest.permission.READ_PHONE_STATE};
        List<String> permissionDeniedList = new ArrayList<>();
        for (String permission : permissions) {
            int permissionCheck = ContextCompat.checkSelfPermission(cordova.getContext(), permission);
            if (permissionCheck == PackageManager.PERMISSION_GRANTED) {
                startListener();
            } else {
                permissionDeniedList.add(permission);
            }
        }
        if (!permissionDeniedList.isEmpty()) {
            String[] deniedPermissions = permissionDeniedList.toArray(new String[permissionDeniedList.size()]);
            ActivityCompat.requestPermissions(cordova.getActivity(), deniedPermissions, 211);
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent intent) {
        super.onActivityResult(requestCode, resultCode, intent);
        if (requestCode == 211 && resultCode == RESULT_OK) {
            startListener();
        }
    }

    private void startListener() {
        if (isFirstListener) {
            isFirstListener = false;
            IntentFilter intentFilter = new IntentFilter();
            intentFilter.addAction(TelephonyManager.ACTION_PHONE_STATE_CHANGED);
            intentFilter.setPriority(Integer.MAX_VALUE);
            cordova.getActivity().registerReceiver(new IncomingReceiver(), intentFilter);
        }
    }

    /**
     * 接收来电广播消息
     */
    public class IncomingReceiver extends BroadcastReceiver {

        private final static String TAG = "IncomingReceiver";
        private String phoneNumber = "";

        @Override
        public void onReceive(Context context, Intent intent) {

            // 如果是拨打电话
            if (intent.getAction().equals(Intent.ACTION_NEW_OUTGOING_CALL)) {
                //拨打电话会优先,收到此广播. 再收到 android.intent.action.PHONE_STATE 的 TelephonyManager.CALL_STATE_OFFHOOK 状态广播;
                String phoneNumber = intent.getStringExtra(Intent.EXTRA_PHONE_NUMBER);
                Log.d(TAG, phoneNumber);//获取拨打的手机号码
            } else {
                // 如果是来电
                TelephonyManager tManager = (TelephonyManager) context
                        .getSystemService(Service.TELEPHONY_SERVICE);
                //电话的状态
                switch (tManager.getCallState()) {
                    case TelephonyManager.CALL_STATE_RINGING:
                        //等待接听状态
                        Log.d(TAG, "TelephonyManager.CALL_STATE_RINGING");
                        phoneNumber = intent.getStringExtra("incoming_number");
                        phoneNumber = numberTrim(phoneNumber);
                        queryUserInfo(phoneNumber);
                        break;
                    case TelephonyManager.CALL_STATE_OFFHOOK:
                        //接听状态
                        Log.d(TAG, "TelephonyManager.CALL_STATE_OFFHOOK");
                        break;
                    case TelephonyManager.CALL_STATE_IDLE:
                        //挂断状态
                        Log.d(TAG, "TelephonyManager.CALL_STATE_IDLE");
                        break;
                }
            }
        }

        /**
         * 电话号码处理
         *
         * @return
         */
        private String numberTrim(String phoneNumber) {
            if (phoneNumber != null && phoneNumber.startsWith("+86"))
                phoneNumber = phoneNumber.substring(3);
            return phoneNumber;
        }

    }

}


